@@ -1,6 +1,8 @@
 pub mod error;
 pub mod extra;
 
+use std::fmt::format;
+use std::io::Write;
 use async_std::fs::*;
 use async_std::io::prelude::*;
 pub use error::*;
@@ -15,8 +17,13 @@ use texcreate_lib::Errors;
 use toml::{from_str, to_string_pretty};
 use zip::write::{FileOptions, ZipWriter};
 use zip::CompressionMethod::Stored;
+use zip::read::ZipFile;
+use crate::TexCreateError::Invalid;
+use rocket::form::FromForm;
 
-#[derive(Debug, Clone, Deserialize, Serialize)]
+type F = std::fs::File;
+
+#[derive(Debug, Clone, Deserialize, Serialize, FromForm)]
 pub struct Config {
     pub author: String,
     pub title: String,
@@ -31,16 +38,6 @@ pub struct Config {
     pub only_files: Option<bool>,
 }
 
-pub type MultiConfig = Vec<Config>;
-
-pub fn to_string_multi(mc: &MultiConfig) -> Result<String, toml::ser::Error> {
-    Ok(to_string_pretty(mc)?)
-}
-
-pub fn from_str_multi(s: String) -> Result<MultiConfig, toml::de::Error> {
-    Ok(from_str(&s)?)
-}
-
 impl Default for Config {
     fn default() -> Self {
         Self {
@@ -118,7 +115,7 @@ impl Config {
     pub fn template(&self) -> TexCreateResult<Latex> {
         let f = match &*self.template {
             "Basic" => basic(
-                self.font_size,
+                self.clone().font_size,
                 &self.paper_size,
                 &self.document_class,
                 &self.author,
@@ -126,7 +123,15 @@ impl Config {
                 &self.date,
             ),
             "Code" => code(
-                self.font_size,
+                self.clone().font_size,
+                &self.paper_size,
+                &self.document_class,
+                &self.author,
+                &self.title,
+                &self.date,
+            ),
+            "Novel" => novel(
+                self.clone().font_size,
                 &self.paper_size,
                 &self.document_class,
                 &self.author,
@@ -179,4 +184,78 @@ impl Config {
         }
         Ok(())
     }
-}
+    pub async fn zip_proj(&self, path: PathBuf) -> TexCreateResult<()>{
+        let mut zip = ZipWriter::new(F::create(format!("{}/{}.zip",path.to_str().unwrap() ,&self.project_name)).unwrap());
+        let options = FileOptions::default().compression_method(Stored);
+
+        // README.md
+        let readme_path = path.join("README.md");
+        let _ = F::create(&readme_path).unwrap();
+        zip.start_file(readme_path.to_str().unwrap(), options).unwrap();
+        zip.write_all(README.as_bytes()).unwrap();
+        // texcreate.toml
+        let toml_path = path.join("texcreate.toml");
+        let _ = F::create(&toml_path).unwrap();
+        zip.start_file(toml_path.to_str().unwrap(), options).unwrap();
+        let mut tex_toml = TexcToml::default();
+        tex_toml.project_name = self.clone().project_name;
+        zip.write_all(to_string_pretty(&tex_toml).unwrap().as_bytes()).unwrap();
+        // out/
+        let out_path = path.join("out");
+        create_dir(&out_path).await?;
+        zip.add_directory(out_path.to_str().unwrap(), options);
+        // src/
+        let src = path.join("src");
+        create_dir(&src).await?;
+
+        // src/*.tex
+        let temp_str = self.template()?.split_string();
+
+        let main = src.join(format!("{}.tex", &self.project_name));
+        let _ = F::create(&main).unwrap();
+
+        let structure = src.join("structure.tex");
+        let _ = F::create(&structure).unwrap();
+
+        zip.start_file(main.to_str().unwrap(), options);
+        zip.write_all(&temp_str.0.as_bytes()).unwrap();
+
+        zip.start_file(structure.to_str().unwrap(), options);
+        zip.write_all(&temp_str.1.as_bytes()).unwrap();
+
+        zip.finish().unwrap();
+
+        // Cleaning step
+        remove_file(&readme_path).await?;
+        remove_file(&toml_path).await?;
+        remove_dir(&out_path).await?;
+        remove_dir_all(&src).await?;
+
+
+        Ok(())
+    }
+    pub async fn zip_files(&self, path: PathBuf) -> TexCreateResult<()>{
+        let mut zip = ZipWriter::new(F::create(format!("{}/{}.zip",path.to_str().unwrap(), &self.project_name)).unwrap());
+        let options = FileOptions::default().compression_method(Stored);
+
+        let main = format!("{}/{}.tex", path.to_str().unwrap(), &self.project_name);
+        let structure = path.join("structure.tex");
+        let _ = File::create(&main);
+        let _ = File::create(&structure);
+        let temp_str = self.template()?.split_string();
+
+        zip.start_file(&main, options).unwrap();
+        zip.write_all(&temp_str.0.as_bytes()).unwrap();
+
+        zip.start_file(structure.to_str().unwrap(), options).unwrap();
+        zip.write_all(&temp_str.1.as_bytes()).unwrap();
+
+        zip.finish().unwrap();
+
+        // Cleaning step
+        remove_file(&main).await?;
+        remove_file(&structure).await?;
+
+        Ok(())
+    }
+}
\ No newline at end of file
